@@ -1,7 +1,7 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2011 INRIA.  All rights reserved.
- * Copyright © 2009-2011 Université Bordeaux 1
+ * Copyright © 2009-2011 inria.  All rights reserved.
+ * Copyright © 2009-2012 Université Bordeaux 1
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
@@ -177,13 +177,15 @@ typedef struct _PSAPI_WORKING_SET_EX_INFORMATION {
 static int
 hwloc_win_set_thread_cpubind(hwloc_topology_t topology __hwloc_attribute_unused, hwloc_thread_t thread, hwloc_const_bitmap_t hwloc_set, int flags)
 {
+  DWORD mask;
+
   if (flags & HWLOC_CPUBIND_NOMEMBIND) {
     errno = ENOSYS;
     return -1;
   }
   /* TODO: groups SetThreadGroupAffinity */
   /* The resulting binding is always strict */
-  DWORD mask = hwloc_bitmap_to_ulong(hwloc_set);
+  mask = hwloc_bitmap_to_ulong(hwloc_set);
   if (!SetThreadAffinityMask(thread, mask))
     return -1;
   return 0;
@@ -219,6 +221,7 @@ hwloc_win_set_thisthread_membind(hwloc_topology_t topology, hwloc_const_nodeset_
 static int
 hwloc_win_set_proc_cpubind(hwloc_topology_t topology __hwloc_attribute_unused, hwloc_pid_t proc, hwloc_const_bitmap_t hwloc_set, int flags)
 {
+  DWORD mask;
   if (flags & HWLOC_CPUBIND_NOMEMBIND) {
     errno = ENOSYS;
     return -1;
@@ -226,7 +229,7 @@ hwloc_win_set_proc_cpubind(hwloc_topology_t topology __hwloc_attribute_unused, h
   /* TODO: groups, hard: has to manually bind all threads into the other group,
    * and the bind the process inside the group */
   /* The resulting binding is always strict */
-  DWORD mask = hwloc_bitmap_to_ulong(hwloc_set);
+  mask = hwloc_bitmap_to_ulong(hwloc_set);
   if (!SetProcessAffinityMask(proc, mask))
     return -1;
   return 0;
@@ -411,39 +414,48 @@ hwloc_win_get_area_membind(hwloc_topology_t topology __hwloc_attribute_unused, c
 {
   SYSTEM_INFO SystemInfo;
   DWORD page_size;
+  uintptr_t start;
+  unsigned nb;
 
   GetSystemInfo(&SystemInfo);
   page_size = SystemInfo.dwPageSize;
 
-  uintptr_t start = (((uintptr_t) addr) / page_size) * page_size;
-  unsigned nb = (((uintptr_t) addr + len - start) + page_size - 1) / page_size;
+  start = (((uintptr_t) addr) / page_size) * page_size;
+  nb = (((uintptr_t) addr + len - start) + page_size - 1) / page_size;
 
   if (!nb)
     nb = 1;
 
   {
-    PSAPI_WORKING_SET_EX_INFORMATION pv[nb];
+    PSAPI_WORKING_SET_EX_INFORMATION *pv;
     unsigned i;
 
+    pv = calloc(nb, sizeof(*pv));
+
     for (i = 0; i < nb; i++)
       pv[i].VirtualAddress = (void*) (start + i * page_size);
-    if (!QueryWorkingSetExProc(GetCurrentProcess(), &pv, sizeof(pv)))
+    if (!QueryWorkingSetExProc(GetCurrentProcess(), pv, nb * sizeof(*pv))) {
+      free(pv);
       return -1;
+    }
     *policy = HWLOC_MEMBIND_BIND;
     if (flags & HWLOC_MEMBIND_STRICT) {
       unsigned node = pv[0].VirtualAttributes.Node;
       for (i = 1; i < nb; i++) {
 	if (pv[i].VirtualAttributes.Node != node) {
 	  errno = EXDEV;
+          free(pv);
 	  return -1;
 	}
       }
       hwloc_bitmap_only(nodeset, node);
+      free(pv);
       return 0;
     }
     hwloc_bitmap_zero(nodeset);
     for (i = 0; i < nb; i++)
       hwloc_bitmap_set(nodeset, pv[i].VirtualAttributes.Node);
+    free(pv);
     return 0;
   }
 }
@@ -520,7 +532,15 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	obj = hwloc_alloc_setup_object(type, id);
         obj->cpuset = hwloc_bitmap_alloc();
 	hwloc_debug("%s#%u mask %lx\n", hwloc_obj_type_string(type), id, procInfo[i].ProcessorMask);
-	hwloc_bitmap_from_ulong(obj->cpuset, procInfo[i].ProcessorMask);
+	/* ProcessorMask is a ULONG_PTR (64/32bits depending on the arch)
+	 * while unsigned long is always 32bits */
+#if SIZEOF_VOID_P == 8
+	hwloc_bitmap_set_ith_ulong(obj->cpuset, 0, procInfo[i].ProcessorMask & 0xffffffff);
+	hwloc_bitmap_set_ith_ulong(obj->cpuset, 1, procInfo[i].ProcessorMask >> 32);
+#else
+	hwloc_bitmap_set_ith_ulong(obj->cpuset, 0, procInfo[i].ProcessorMask);
+#endif
+	hwloc_debug_2args_bitmap("%s#%u bitmap %s\n", hwloc_obj_type_string(type), id, obj->cpuset);
 
 	switch (type) {
 	  case HWLOC_OBJ_NODE:
@@ -579,8 +599,8 @@ hwloc_look_windows(struct hwloc_topology *topology)
       }
 
       for (procInfo = procInfoTotal;
-	   (void*) procInfo < (void*) ((unsigned long) procInfoTotal + length);
-	   procInfo = (void*) ((unsigned long) procInfo + procInfo->Size)) {
+	   (void*) procInfo < (void*) ((uintptr_t) procInfoTotal + length);
+	   procInfo = (void*) ((uintptr_t) procInfo + procInfo->Size)) {
         unsigned num, i;
         GROUP_AFFINITY *GroupMask;
 
@@ -622,7 +642,15 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	      mask = procInfo->Group.GroupInfo[id].ActiveProcessorMask;
 	      hwloc_debug("group %u %d cpus mask %lx\n", id,
                   procInfo->Group.GroupInfo[id].ActiveProcessorCount, mask);
-	      hwloc_bitmap_from_ith_ulong(obj->cpuset, id, mask);
+	      /* KAFFINITY is ULONG_PTR (64/32bits depending on the arch)
+	       * while unsigned long is always 32bits */
+#if SIZEOF_VOID_P == 8
+	      hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*id, mask & 0xffffffff);
+	      hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*id+1, mask >> 32);
+#else
+	      hwloc_bitmap_set_ith_ulong(obj->cpuset, id, mask);
+#endif
+	      hwloc_debug_2args_bitmap("group %u %d bitmap %s\n", id, procInfo->Group.GroupInfo[id].ActiveProcessorCount, obj->cpuset);
 	      hwloc_insert_object_by_cpuset(topology, obj);
 	    }
 	    continue;
@@ -636,8 +664,16 @@ hwloc_look_windows(struct hwloc_topology *topology)
         obj->cpuset = hwloc_bitmap_alloc();
         for (i = 0; i < num; i++) {
           hwloc_debug("%s#%u %d: mask %d:%lx\n", hwloc_obj_type_string(type), id, i, GroupMask[i].Group, GroupMask[i].Mask);
-          hwloc_bitmap_from_ith_ulong(obj->cpuset, GroupMask[i].Group, GroupMask[i].Mask);
+	  /* GROUP_AFFINITY.Mask is KAFFINITY, which is ULONG_PTR (64/32bits depending on the arch)
+	   * while unsigned long is always 32bits */
+#if SIZEOF_VOID_P == 8
+          hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*GroupMask[i].Group, GroupMask[i].Mask & 0xffffffff);
+          hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*GroupMask[i].Group+1, GroupMask[i].Mask >> 32);
+#else
+          hwloc_bitmap_set_ith_ulong(obj->cpuset, GroupMask[i].Group, GroupMask[i].Mask);
+#endif
         }
+	hwloc_debug("%s#%u bitmap %s\n", hwloc_obj_type_string(type), id, obj->cpuset);
 
 	switch (type) {
 	  case HWLOC_OBJ_NODE:
