@@ -207,13 +207,14 @@ static void look_proc(struct procinfo *infos, unsigned highest_cpuid, unsigned h
 
       cache->linesize = linesize = (ebx & 0xfff) + 1;
       cache->linepart = linepart = ((ebx >> 12) & 0x3ff) + 1;
+      ways = ((ebx >> 22) & 0x3ff) + 1;
       if (eax & (1 << 9))
         /* Fully associative */
         cache->ways = -1;
       else
-        cache->ways = ways = ((ebx >> 22) & 0x3ff) + 1;
+        cache->ways = ways;
       cache->sets = sets = ecx + 1;
-      cache->size = linesize * linepart * ways * sets; /* FIXME: what if ways == -1 ? */
+      cache->size = linesize * linepart * ways * sets;
 
       hwloc_debug("cache %u type %u L%u t%u c%u linesize %u linepart %u ways %u sets %u, size %uKB\n", cachenum, cache->type, cache->level, cache->nbthreads_sharing, infos->max_nbcores, linesize, linepart, ways, sets, cache->size >> 10);
       infos->max_nbthreads = infos->max_log_proc / infos->max_nbcores;
@@ -321,7 +322,7 @@ static void summarize(hwloc_topology_t topology, struct procinfo *infos, unsigne
       if (infos[one].otherids[level] != UINT_MAX) {
 	hwloc_bitmap_t unknowns_cpuset = hwloc_bitmap_dup(complete_cpuset);
 	hwloc_bitmap_t unknown_cpuset;
-	hwloc_obj_t unknown;
+	hwloc_obj_t unknown_obj;
 
 	while ((i = hwloc_bitmap_first(unknowns_cpuset)) != (unsigned) -1) {
 	  unsigned unknownid = infos[i].otherids[level];
@@ -333,12 +334,12 @@ static void summarize(hwloc_topology_t topology, struct procinfo *infos, unsigne
 	      hwloc_bitmap_clr(unknowns_cpuset, j);
 	    }
 	  }
-	  unknown = hwloc_alloc_setup_object(HWLOC_OBJ_MISC, unknownid);
-	  unknown->cpuset = unknown_cpuset;
-	  unknown->os_level = level;
+	  unknown_obj = hwloc_alloc_setup_object(HWLOC_OBJ_MISC, unknownid);
+	  unknown_obj->cpuset = unknown_cpuset;
+	  unknown_obj->os_level = level;
 	  hwloc_debug_2args_bitmap("os unknown%d %u has cpuset %s\n",
 	      level, unknownid, unknown_cpuset);
-	  hwloc_insert_object_by_cpuset(topology, unknown);
+	  hwloc_insert_object_by_cpuset(topology, unknown_obj);
 	}
 	hwloc_bitmap_free(unknowns_cpuset);
       }
